"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function isRequireCall(path) {
    const callee = path.get("callee");
    return (callee.isIdentifier() && callee.node.name === "require" && !callee.scope.getBinding("require"));
}
function getReferencedNamedImport(identifier, expectedNameAsExported) {
    const binding = identifier.scope.getBinding(identifier.node.name);
    if (!binding)
        return;
    const bindingPath = binding.path;
    if (bindingPath.isImportSpecifier() &&
        bindingPath.get("imported").node.name === expectedNameAsExported) {
        // import { sql } from "..."
        return bindingPath.get("imported");
    }
    if (bindingPath.isVariableDeclarator() && bindingPath.get("init").isCallExpression()) {
        const init = bindingPath.get("init");
        if (isRequireCall(init) && bindingPath.get("id").isObjectPattern()) {
            // const { sql } = require("...")
            const destructuring = bindingPath.get("id");
            for (const property of destructuring.get("properties")) {
                if (property.isObjectProperty() &&
                    !Array.isArray(property.get("key")) &&
                    property.get("key").isIdentifier() &&
                    property.get("key").node.name === expectedNameAsExported) {
                    return property.get("key");
                }
            }
        }
    }
    if (bindingPath.isVariableDeclarator() &&
        bindingPath.get("id").isIdentifier() &&
        bindingPath.get("init").isMemberExpression()) {
        const id = bindingPath.get("id");
        const init = bindingPath.get("init");
        const initObject = init.get("object");
        const initProp = init.get("property");
        if (id.node.name === binding.identifier.name &&
            initObject.isCallExpression() &&
            isRequireCall(initObject)) {
            if (!Array.isArray(initProp) &&
                initProp.isIdentifier() &&
                initProp.node.name === expectedNameAsExported) {
                return id;
            }
            else if (!Array.isArray(initProp) &&
                initProp.isStringLiteral() &&
                initProp.node.value === expectedNameAsExported) {
                return id;
            }
        }
    }
    return undefined;
}
exports.getReferencedNamedImport = getReferencedNamedImport;
