"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const fs = __importStar(require("fs"));
const errors_1 = require("../errors");
const file_1 = require("../typescript/file");
const parse_query_1 = require("./parse-query");
const babel_imports_1 = require("./babel-imports");
const parse_query_invocation_1 = require("./parse-query-invocation");
const parse_table_definition_1 = require("./parse-table-definition");
function compileTypeScript(filePath) {
    try {
        return file_1.compileFiles([filePath]);
    }
    catch (error) {
        // tslint:disable-next-line no-console
        console.error(`Compiling TypeScript source file ${filePath} failed: ${error.message}`);
    }
}
function loadSourceFile(filePath) {
    const fileContent = fs.readFileSync(filePath, "utf8");
    const program = filePath.endsWith(".ts") ? compileTypeScript(filePath) : undefined;
    return {
        fileContent,
        filePath,
        ts: program
            ? {
                program,
                sourceFile: program.getSourceFile(filePath) ||
                    errors_1.fail(`Cannot retrieve source file ${filePath} from compiled TypeScript sources.`)
            }
            : undefined
    };
}
exports.loadSourceFile = loadSourceFile;
function parseSourceFile(sourceFile) {
    const queries = [];
    const tableSchemas = [];
    const ast = core_1.parse(sourceFile.fileContent, {
        filename: sourceFile.filePath,
        plugins: sourceFile.ts ? ["@babel/plugin-transform-typescript"] : [],
        sourceType: "unambiguous"
    });
    core_1.traverse(ast, {
        CallExpression(path) {
            const callee = path.get("callee");
            if (!callee.isIdentifier())
                return;
            const importSpecifier = babel_imports_1.getReferencedNamedImport(callee, "defineTable");
            if (!importSpecifier)
                return;
            tableSchemas.push(parse_table_definition_1.parseTableDefinition(path, sourceFile));
        },
        TaggedTemplateExpression(path) {
            const tag = path.get("tag");
            if (!tag.isIdentifier())
                return;
            const importSpecifier = babel_imports_1.getReferencedNamedImport(tag, "sql");
            if (!importSpecifier)
                return;
            const query = parse_query_1.parseQuery(path.get("quasi"), sourceFile);
            queries.push(parse_query_invocation_1.createQueryInvocation(query, path, sourceFile));
        }
    });
    return {
        queries,
        tableSchemas
    };
}
exports.parseSourceFile = parseSourceFile;
