"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const format = __importStar(require("../format"));
const file_1 = require("../typescript/file");
const objectish_1 = require("../typescript/objectish");
function resolveTypeParamPropTypes(typeParameters, tsProgram, tsSource) {
    if (typeParameters.node) {
        const params = typeParameters.get("params");
        const paramTypes = Array.isArray(params) ? params : [params];
        if (paramTypes.length === 1) {
            const type = file_1.resolveTypeOfBabelPath(paramTypes[0].node, tsProgram, tsSource);
            const propTypes = type ? objectish_1.resolvePropertyTypes(tsProgram, type) : null;
            return propTypes;
        }
    }
    return null;
}
function createQueryInvocation(query, path, sourceFile) {
    let resultTypeAssertion;
    if (path.parentPath.isCallExpression() && sourceFile.ts) {
        const callExpression = path.parentPath;
        const typeParameters = callExpression.get("typeParameters");
        const propTypes = resolveTypeParamPropTypes(typeParameters, sourceFile.ts.program, sourceFile.ts.sourceFile);
        if (propTypes && Object.keys(propTypes).length > 0) {
            const schema = objectish_1.mapPropertyTypesToSchemaDescriptor(propTypes);
            resultTypeAssertion = {
                path: typeParameters,
                schema
            };
        }
        else if (typeParameters.node) {
            const { loc } = callExpression.node;
            console.warn(format.warning(`Warning: Cannot infer type of type parameter. Skipping type checking of this expression.\n` +
                `  File: ${sourceFile.filePath}${loc ? `:${loc.start.line}` : ""}\n` +
                `  Type parameter: ${typeParameters.getSource()}`));
        }
        else {
            // No type params specified - Ignore
        }
    }
    return {
        query,
        resultTypeAssertion
    };
}
exports.createQueryInvocation = createQueryInvocation;
