"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const babel_imports_1 = require("./babel-imports");
const format = __importStar(require("../format"));
const parse_pg_query_1 = require("../postgres/parse-pg-query");
const file_1 = require("../typescript/file");
const objectish_1 = require("../typescript/objectish");
function isSpreadCallExpression(expression, fnName) {
    if (!expression.isCallExpression())
        return false;
    const callee = expression.get("callee");
    if (!callee.isIdentifier())
        return false;
    const importSpecifier = babel_imports_1.getReferencedNamedImport(callee, fnName);
    return Boolean(importSpecifier);
}
function resolveSpreadArgumentType(expression, tsProgram, tsSource) {
    const callee = expression.get("callee");
    const args = expression.get("arguments");
    if (args.length === 0) {
        throw new Error(`Expected call to ${callee.node.name}() to have arguments.`);
    }
    const spreadArg = args[0];
    if (!spreadArg.node.start || !spreadArg.node.end)
        return null;
    return file_1.resolveTypeOfBabelPath(spreadArg.node, tsProgram, tsSource);
}
function resolveSpreadExpressionType(expression, sourceFile) {
    if (!sourceFile.ts)
        return parse_pg_query_1.spreadTypeAny;
    if (isSpreadCallExpression(expression, "spreadInsert")) {
        const spreadArgType = resolveSpreadArgumentType(expression, sourceFile.ts.program, sourceFile.ts.sourceFile);
        const spreadType = spreadArgType
            ? objectish_1.resolvePropertyTypes(sourceFile.ts.program, spreadArgType)
            : null;
        return spreadType || parse_pg_query_1.spreadTypeAny;
    }
    else {
        return null;
    }
}
function parseQuery(path, sourceFile) {
    const expressions = path.get("expressions");
    const textPartials = path.get("quasis").map(quasi => quasi.node);
    const expressionSpreadTypes = {};
    const sourceMap = [];
    let templatedQueryString = "";
    const addToQueryString = (node, queryStringPartial, isTemplateExpression) => {
        if (node.loc) {
            sourceMap.push({
                isTemplateExpression,
                sourceLocation: node.loc,
                queryStartIndex: templatedQueryString.length,
                queryEndIndex: templatedQueryString.length + queryStringPartial.length
            });
        }
        return templatedQueryString + queryStringPartial;
    };
    templatedQueryString = addToQueryString(textPartials[0], textPartials[0].value.cooked);
    for (let index = 0; index < expressions.length; index++) {
        const expression = expressions[index];
        const paramNumber = index + 1;
        const placeholder = isSpreadCallExpression(expression, "spreadInsert")
            ? `SELECT \$${paramNumber}`
            : `\$${paramNumber}`;
        templatedQueryString = addToQueryString(expression.node, placeholder, true);
        expressionSpreadTypes[paramNumber] = resolveSpreadExpressionType(expression, sourceFile);
        if (expressionSpreadTypes[paramNumber] === parse_pg_query_1.spreadTypeAny && sourceFile.ts) {
            const lineHint = path.node.loc ? `:${path.node.loc.start.line}` : ``;
            console.warn(format.warning(`Warning: Cannot infer properties of spread expression in SQL template at ${sourceFile.filePath}${lineHint}`));
        }
        if (textPartials[index + 1]) {
            templatedQueryString = addToQueryString(textPartials[index + 1], textPartials[index + 1].value.cooked);
        }
    }
    return parse_pg_query_1.parsePostgresQuery(templatedQueryString, sourceFile, sourceMap, expressionSpreadTypes);
}
exports.parseQuery = parseQuery;
