"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const babel_resolver_1 = require("./babel-resolver");
function parseColumnDescriptors(path) {
    const columnDescriptors = {};
    const resolved = babel_resolver_1.resolveColumnDescriptorExpression(path);
    if (!resolved || typeof resolved !== "object") {
        throw path.buildCodeFrameError(`Statically resolved to ${typeof resolved}. Expected it to be an object.`);
    }
    for (const key of Object.keys(resolved)) {
        const value = resolved[key];
        if (!babel_resolver_1.isDescriptor(value)) {
            throw path.buildCodeFrameError(`Expected property "${key}" to be a schema descriptor.`);
        }
        columnDescriptors[key] = value;
    }
    return columnDescriptors;
}
function parseTableDefinition(path, sourceFile) {
    const args = path.get("arguments");
    if (args.length !== 2) {
        throw path.buildCodeFrameError("Expected two arguments on defineTable() call.");
    }
    const tableName = args[0].evaluate().value;
    if (!tableName || typeof tableName !== "string") {
        throw path.buildCodeFrameError("Expected first argument of defineTable() to be the table name.");
    }
    const tableColumnDefs = args[1];
    if (!tableColumnDefs.isObjectExpression()) {
        throw path.buildCodeFrameError("Second argument to defineTable() must be an object literal.");
    }
    const columnDescriptors = parseColumnDescriptors(tableColumnDefs);
    return {
        tableName,
        columnDescriptors,
        columnNames: Object.keys(columnDescriptors),
        loc: path.node.loc,
        sourceFile
    };
}
exports.parseTableDefinition = parseTableDefinition;
