"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const utils_1 = require("./utils");
const parse_file_1 = require("./babel/parse-file");
exports.loadSourceFile = parse_file_1.loadSourceFile;
const debugFile = debug_1.default("postguard:file");
const debugQueries = debug_1.default("postguard:query");
const debugSubqueries = debug_1.default("postguard:subquery");
const debugTables = debug_1.default("postguard:table");
function formatColumnRefs(columnRefs) {
    const formattedColumnRefs = columnRefs.map(col => "tableName" in col ? `${col.tableName}.${col.columnName}` : col.columnName);
    return formattedColumnRefs.length > 0 ? formattedColumnRefs.join(", ") : "-";
}
function stringifyColumnType(descriptor) {
    const props = [
        descriptor.hasDefault ? "default value" : null,
        descriptor.nullable ? "nullable" : null
    ].filter(str => !!str);
    const propsString = props.length > 0 ? ` (${props.join(", ")})` : "";
    if (descriptor.type === "enum" && descriptor.enum) {
        return `enum${propsString} [${descriptor.enum.map(value => `'${value}'`).join(", ")}]`;
    }
    else {
        return `${descriptor.type}${descriptor.subtype ? `[${descriptor.subtype.type}]` : ""}${propsString}`;
    }
}
function parseSourceFile(sourceFile) {
    debugFile(`Start parsing file ${sourceFile.filePath}`);
    const { queries, tableSchemas } = parse_file_1.parseSourceFile(sourceFile);
    debugFile(`Parsed file ${sourceFile.filePath}:`);
    for (const invocation of queries) {
        const query = invocation.query;
        debugQueries(`  Query: ${query.query.trim()}\n` +
            `    Result columns: ${formatColumnRefs(query.returnedColumns)}\n` +
            `    Referenced columns: ${formatColumnRefs(query.referencedColumns)}`);
        for (const subquery of utils_1.getAllSubqueries(query)) {
            const returningStatus = subquery.returnsIntoParentQuery ? " (into parent query)" : "";
            debugSubqueries(`    Subquery type: ${subquery.path.type}\n` +
                `      Result columns: ${formatColumnRefs(subquery.returnedColumns)}${returningStatus}\n` +
                `      Referenced columns: ${formatColumnRefs(subquery.referencedColumns)}`);
        }
    }
    for (const table of tableSchemas) {
        debugTables(`  Table: ${table.tableName}`);
        for (const columnName of table.columnNames) {
            const columnType = stringifyColumnType(table.columnDescriptors[columnName]);
            debugTables(`    Column "${columnName}": ${columnType}`);
        }
    }
    return { queries, tableSchemas };
}
exports.parseSourceFile = parseSourceFile;
