"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pg_node_types_1 = require("./pg-node-types");
const $cancelToken = Symbol("cancel");
function createQueryNodePath(node, ancestors, parentPropKey) {
    return {
        ancestors,
        node,
        parentPropKey,
        type: pg_node_types_1.getNodeType(node)
    };
}
exports.createQueryNodePath = createQueryNodePath;
function createQueryNodeSubpath(path, subNode, parentPropKey) {
    return {
        ancestors: [...path.ancestors, path],
        node: subNode,
        parentPropKey,
        type: pg_node_types_1.getNodeType(subNode)
    };
}
exports.createQueryNodeSubpath = createQueryNodeSubpath;
function traverseArray(path, array, parentPropKey, callback) {
    for (const item of array) {
        if (!item)
            continue;
        if (Array.isArray(item)) {
            traverseArray(path, item, parentPropKey, callback);
        }
        else {
            const itemPath = createQueryNodeSubpath(path, item, parentPropKey);
            traverseSubTree(itemPath, callback);
        }
    }
}
function traverseSubTree(path, callback) {
    const callbackResult = callback(path, $cancelToken);
    if (callbackResult === $cancelToken)
        return;
    for (const key of Object.keys(path.node[path.type])) {
        const propValue = path.node[path.type][key];
        if (Array.isArray(propValue)) {
            traverseArray(path, propValue, key, callback);
        }
        else if (propValue && typeof propValue === "object") {
            const propPath = createQueryNodeSubpath(path, propValue, key);
            traverseSubTree(propPath, callback);
        }
    }
}
exports.traverseSubTree = traverseSubTree;
function getQueryPathParent(path) {
    return path.ancestors.length >= 1 ? path.ancestors[path.ancestors.length - 1] : null;
}
exports.getQueryPathParent = getQueryPathParent;
