"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const format = __importStar(require("../format"));
function compileFiles(filePaths) {
    return typescript_1.default.createProgram({
        rootNames: filePaths,
        options: {
            noEmit: true,
            skipLibCheck: true,
            strict: false
        }
    });
}
exports.compileFiles = compileFiles;
function getNodeAtPosition(nodeToSearch, startPosition, endPosition) {
    let resultNode = null;
    typescript_1.default.forEachChild(nodeToSearch, node => {
        if (node.pos === startPosition && node.end === endPosition) {
            resultNode = node;
            return true;
        }
        else if (node.pos <= startPosition && node.end >= endPosition) {
            const recursionResult = getNodeAtPosition(node, startPosition, endPosition);
            if (recursionResult) {
                resultNode = recursionResult;
                return true;
            }
        }
        if (node.pos > startPosition) {
            return true;
        }
    });
    return resultNode;
}
function resolveTypeOfBabelPath(babelNode, program, source) {
    if (!babelNode.start || !babelNode.end) {
        console.warn(format.warning(`Warning: Could not match SQL template string expression node between Babel and TypeScript parser. Skipping type checking of this expression.\n` +
            `  File: ${source.fileName}`));
        return null;
    }
    const node = getNodeAtPosition(source, babelNode.start, babelNode.end);
    if (!node) {
        console.warn(format.warning(`Warning: Could not match SQL template string expression node between Babel and TypeScript parser. Skipping type checking of this expression.\n` +
            `  File: ${source.fileName}\n` +
            `  Template expression: ${source.getText().substring(babelNode.start, babelNode.end)}`));
        return null;
    }
    const checker = program.getTypeChecker();
    const type = checker.getTypeAtLocation(node);
    if (!type) {
        console.warn(format.warning(`Warning: Could not resolve TypeScript type for SQL template string expression. Skipping type checking of this expression.\n` +
            `  File: ${source.fileName}\n` +
            `  Template expression: ${source.getText().substring(babelNode.start, babelNode.end)}`));
        return null;
    }
    return type;
}
exports.resolveTypeOfBabelPath = resolveTypeOfBabelPath;
