"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const squid_1 = require("squid");
const typescript_1 = __importDefault(require("typescript"));
const isObjectType = (type) => Boolean(type.flags & typescript_1.default.TypeFlags.Object);
const getTypeElementName = (element) => {
    const name = element.name;
    if (!name)
        return null;
    if (typescript_1.default.isComputedPropertyName(name)) {
        return null;
    }
    else {
        return name.text;
    }
};
function getObjectTypeProperties(program, type) {
    const checker = program.getTypeChecker();
    const typeNode = checker.typeToTypeNode(type);
    const members = typeNode && typeNode.kind === typescript_1.default.SyntaxKind.TypeLiteral
        ? typeNode.members
        : typescript_1.default.createNodeArray([]);
    return type.getProperties().reduce((propertiesObject, propertySymbol) => {
        const propTypeNode = members.find(member => getTypeElementName(member) === propertySymbol.name);
        const propType = propTypeNode ? propTypeNode.type : null;
        return Object.assign({}, propertiesObject, { [propertySymbol.getName()]: propType });
    }, {});
}
function resolvePropertyTypes(program, type) {
    if (type.isIntersection()) {
        return type.types.reduce(
        // TODO: Don't just override previous property type
        (reduced, subType) => (Object.assign({}, reduced, resolvePropertyTypes(program, subType))), {});
    }
    if (!isObjectType(type))
        return null;
    const inheritedTypes = type.objectFlags & typescript_1.default.ObjectFlags.Interface
        ? type.getBaseTypes() || []
        : [];
    const inheritedObjectTypes = inheritedTypes.filter(isObjectType);
    return [...inheritedObjectTypes, type].reduce((propertiesObject, someType) => (Object.assign({}, propertiesObject, getObjectTypeProperties(program, someType))), {});
}
exports.resolvePropertyTypes = resolvePropertyTypes;
function mapPropertyTypesToSchemaDescriptor(propTypes) {
    // TODO: Map types properly
    const schema = {};
    for (const key of Object.keys(propTypes)) {
        schema[key] = squid_1.Schema.Any;
    }
    return schema;
}
exports.mapPropertyTypesToSchemaDescriptor = mapPropertyTypesToSchemaDescriptor;
